"""Added column aggregated_at to CVE

Revision ID: 9b210f8d5c24
Revises: 79e1d320db3d
Create Date: 2019-12-06 19:08:16.125613+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9b210f8d5c24"
down_revision = "79e1d320db3d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("cve", sa.Column("aggregated_at", sa.DateTime(), nullable=True))
    op.alter_column("cve", "cve_id", existing_type=sa.VARCHAR(length=256), nullable=False)
    op.create_unique_constraint(None, "cve", ["cve_id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "cve", type_="unique")
    op.alter_column("cve", "cve_id", existing_type=sa.VARCHAR(length=256), nullable=True)
    op.drop_column("cve", "aggregated_at")
    # ### end Alembic commands ###



"""add is_external to PackageExtractRun

Revision ID: 079e1d92f9d4
Revises: 650a13fd0a08
Create Date: 2019-10-08 12:33:51.979019+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "079e1d92f9d4"
down_revision = "650a13fd0a08"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("has_symbol", sa.Column("external_software_environment_id", sa.Integer(), nullable=False))
    op.create_foreign_key(
        None,
        "has_symbol",
        "external_software_environment",
        ["external_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    op.add_column("package_extract_run", sa.Column("external_software_environment_id", sa.Integer(), nullable=False))
    op.create_foreign_key(
        None,
        "package_extract_run",
        "external_software_environment",
        ["external_software_environment_id"],
        ["id"],
        ondelete="CASCADE",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "package_extract_run", type_="foreignkey")
    op.drop_column("package_extract_run", "external_software_environment_id")
    op.drop_constraint(None, "has_symbol", type_="foreignkey")
    op.drop_column("has_symbol", "external_software_environment_id")
    # ### end Alembic commands ###



"""Extend Python package information stored in the database

Revision ID: 95c40432360f
Revises: 894ddaef0c43
Create Date: 2021-07-01 12:51:03.294947+00:00

"""
from alembic import op
from sqlalchemy.sql import text
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "95c40432360f"
down_revision = "894ddaef0c43"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    conn = op.get_bind()
    conn.execute(text("DELETE FROM identified"))
    op.add_column("identified", sa.Column("location", sa.Text(), nullable=False))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("identified", "location")
    # ### end Alembic commands ###



#!/usr/bin/env python3
# thoth-storages
# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
#
# This program is free software: you can redistribute it and / or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Adapter for storing build logs."""

import hashlib
import os
import typing

from .ceph import CephStore
from .base import StorageBase


class BuildLogsStore(StorageBase):
    """Adapter for storing build logs."""

    RESULT_TYPE = "buildlogs"

    def __init__(
        self,
        deployment_name=None,
        *,
        bucket_prefix: str = None,
        host: str = None,
        key_id: str = None,
        secret_key: str = None,
        bucket: str = None,
        region: str = None,
    ):
        """Initialize adapter for storing build logs.

        Parameters not explicitly provided will be picked from env variables.
        """
        self.deployment_name = deployment_name or os.environ["THOTH_DEPLOYMENT_NAME"]
        self.prefix = "{}/{}/{}/".format(
            bucket_prefix or os.environ["THOTH_CEPH_BUCKET_PREFIX"], self.deployment_name, self.RESULT_TYPE
        )
        self.ceph = CephStore(
            self.prefix, host=host, key_id=key_id, secret_key=secret_key, bucket=bucket, region=region
        )

    def is_connected(self) -> bool:
        """Check if the given database adapter is in connected state."""
        return self.ceph.is_connected()

    def connect(self) -> None:
        """Connect the given storage adapter."""
        self.ceph.connect()

    def store_document(self, document: dict) -> str:
        """Store the given document in Ceph."""
        blob = self.ceph.dict2blob(document)
        document_id = "buildlog-" + hashlib.sha256(blob).hexdigest()
        self.ceph.store_blob(blob, document_id)
        return document_id

    def retrieve_document(self, document_id: str) -> dict:
        """Retrieve a document from Ceph by its id."""
        return self.ceph.retrieve_document(document_id)

    def iterate_results(self) -> typing.Generator[tuple, None, None]:
        """Iterate over results available in the Ceph."""
        return self.ceph.iterate_results()

    def get_document_listing(self) -> typing.Generator[str, None, None]:
        """Get listing of documents stored on the Ceph."""
        return self.ceph.get_document_listing()



"""Allow nullable references for software environments

Revision ID: 54d50b0eeb33
Revises: e05d6d95ea95
Create Date: 2019-10-21 11:54:17.054303+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "54d50b0eeb33"
down_revision = "e05d6d95ea95"
branch_labels = None
depends_on = None


def upgrade():
    op.drop_constraint("has_symbol_pkey", "has_symbol", type_="primary")
    op.create_primary_key(
        "has_symbol_pkey",
        "has_symbol",
        [
            "id",
        ],
    )
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("has_symbol", "external_software_environment_id", existing_type=sa.INTEGER(), nullable=True)
    op.alter_column("has_symbol", "software_environment_id", existing_type=sa.INTEGER(), nullable=True)
    op.alter_column("has_symbol", "versioned_symbol_id", existing_type=sa.INTEGER(), nullable=True)
    # ### end Alembic commands ###


def downgrade():
    op.drop_constraint("has_symbol_pkey", "has_symbol", type_="primary")
    op.create_primary_key("has_symbol_pkey", "has_symbol", ["id", "software_environment_id"])
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("has_symbol", "versioned_symbol_id", existing_type=sa.INTEGER(), nullable=False)
    op.alter_column("has_symbol", "software_environment_id", existing_type=sa.INTEGER(), nullable=False)
    op.alter_column("has_symbol", "external_software_environment_id", existing_type=sa.INTEGER(), nullable=False)
    # ### end Alembic commands ###



"""Add markers and extra needed for resolver

Revision ID: f60593780969
Revises: 1ee67d9a7861
Create Date: 2019-10-02 20:04:59.209567+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "f60593780969"
down_revision = "1ee67d9a7861"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("depends_on", sa.Column("extra", sa.String(length=256), nullable=True))
    op.add_column("depends_on", sa.Column("marker", sa.String(length=256), nullable=True))
    op.add_column("depends_on", sa.Column("marker_evaluation_result", sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("depends_on", "marker_evaluation_result")
    op.drop_column("depends_on", "marker")
    op.drop_column("depends_on", "extra")
    # ### end Alembic commands ###



"""Initial schema

Revision ID: 9168881dd125
Revises:
Create Date: 2019-09-19 11:56:44.670642+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "9168881dd125"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "cve",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("advisory", sa.String(length=16384), nullable=True),
        sa.Column("cve_name", sa.String(length=256), nullable=True),
        sa.Column("cve_id", sa.String(length=256), nullable=True),
        sa.Column("version_range", sa.String(length=256), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "deb_dependency",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_name", sa.String(length=256), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "deb_package_version",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_name", sa.String(length=256), nullable=False),
        sa.Column("package_version", sa.String(length=256), nullable=False),
        sa.Column("epoch", sa.String(length=256), nullable=True),
        sa.Column("arch", sa.String(length=256), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "ecosystem_solver",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("ecosystem", sa.String(length=256), nullable=False),
        sa.Column("solver_name", sa.String(length=256), nullable=False),
        sa.Column("solver_version", sa.String(length=16), nullable=False),
        sa.Column("os_name", sa.String(length=128), nullable=False),
        sa.Column("os_version", sa.String(length=8), nullable=False),
        sa.Column("python_version", sa.String(length=8), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("ecosystem", "solver_name", "solver_version", "os_name", "os_version", "python_version"),
    )
    op.create_index(
        "ecosystem_solver_idx",
        "ecosystem_solver",
        ["ecosystem", "solver_name", "solver_version", "os_name", "os_version", "python_version"],
        unique=True,
    )
    op.create_table(
        "hardware_information",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("cpu_vendor", sa.Integer(), nullable=True),
        sa.Column("cpu_model", sa.Integer(), nullable=True),
        sa.Column("cpu_cores", sa.Integer(), nullable=True),
        sa.Column("cpu_model_name", sa.String(length=256), nullable=True),
        sa.Column("cpu_family", sa.Integer(), nullable=True),
        sa.Column("cpu_physical_cpus", sa.Integer(), nullable=True),
        sa.Column("gpu_model_name", sa.String(length=256), nullable=True),
        sa.Column("gpu_vendor", sa.String(length=256), nullable=True),
        sa.Column("gpu_cores", sa.Integer(), nullable=True),
        sa.Column("gpu_memory_size", sa.Integer(), nullable=True),
        sa.Column("ram_size", sa.Integer(), nullable=True),
        sa.Column("is_user", sa.Boolean(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_artifact",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("artifact_hash_sha256", sa.String(length=256), nullable=False),
        sa.Column("artifact_name", sa.String(length=256), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_file_digest",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("sha256", sa.String(length=256), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("sha256"),
    )
    op.create_index("sha256_idx", "python_file_digest", ["sha256"], unique=True)
    op.create_table(
        "python_package_index",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("url", sa.String(length=256), nullable=False),
        sa.Column("warehouse_api_url", sa.String(length=256), nullable=True),
        sa.Column("verify_ssl", sa.Boolean(), nullable=False),
        sa.Column("enabled", sa.Boolean(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("url"),
    )
    op.create_index("url_idx", "python_package_index", ["url"], unique=True)
    op.create_table(
        "python_software_stack",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column(
            "software_stack_type",
            postgresql.ENUM("USER", "INSPECTION", "ADVISED", name="software_stack_type"),
            nullable=True,
        ),
        sa.Column("performance_score", sa.Float(), nullable=True),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "rpm_package_version",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_name", sa.String(length=256), nullable=False),
        sa.Column("package_version", sa.String(length=256), nullable=False),
        sa.Column("release", sa.String(length=256), nullable=True),
        sa.Column("epoch", sa.String(length=256), nullable=True),
        sa.Column("arch", sa.String(length=256), nullable=True),
        sa.Column("src", sa.Boolean(), nullable=True),
        sa.Column("package_identifier", sa.String(length=256), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "rpm_requirement",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("rpm_requirement_name", sa.String(length=256), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "software_environment",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("environment_name", sa.String(length=256), nullable=True),
        sa.Column("python_version", sa.String(length=256), nullable=True),
        sa.Column("image_name", sa.String(length=256), nullable=True),
        sa.Column("image_sha", sa.String(length=256), nullable=True),
        sa.Column("os_name", sa.String(length=256), nullable=True),
        sa.Column("os_version", sa.String(length=256), nullable=True),
        sa.Column("cuda_version", sa.String(length=256), nullable=True),
        sa.Column("environment_type", postgresql.ENUM("RUNTIME", "BUILDTIME", name="environment_type"), nullable=False),
        sa.Column("is_user", sa.Boolean(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "versioned_symbol",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("library_name", sa.String(length=256), nullable=False),
        sa.Column("symbol", sa.String(length=256), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "adviser_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("adviser_document_id", sa.String(length=256), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("adviser_version", sa.String(length=256), nullable=False),
        sa.Column("adviser_name", sa.String(length=256), nullable=False),
        sa.Column("count", sa.Integer(), nullable=True),
        sa.Column("limit", sa.Integer(), nullable=True),
        sa.Column("origin", sa.String(length=256), nullable=True),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("limit_latest_versions", sa.Integer(), nullable=False),
        sa.Column("adviser_error", sa.Boolean(), nullable=False),
        sa.Column(
            "recommendation_type",
            postgresql.ENUM("STABLE", "TESTING", "LATEST", name="recommendation_type"),
            nullable=False,
        ),
        sa.Column("requirements_format", postgresql.ENUM("PIPENV", name="requirements_format"), nullable=False),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("advised_configuration_changes", sa.Boolean(), nullable=False),
        sa.Column("additional_stack_info", sa.Boolean(), nullable=False),
        sa.Column("user_run_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("user_build_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("user_software_stack_id", sa.Integer(), nullable=True),
        sa.Column("hardware_information_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["user_build_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["user_run_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["user_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "deb_depends",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("version_range", sa.String(length=256), nullable=False),
        sa.Column("deb_dependency_id", sa.Integer(), nullable=False),
        sa.Column("deb_package_version_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["deb_dependency_id"], ["deb_dependency.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["deb_package_version_id"], ["deb_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "deb_dependency_id", "deb_package_version_id"),
    )
    op.create_table(
        "deb_pre_depends",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("deb_dependency_id", sa.Integer(), nullable=False),
        sa.Column("deb_package_version_id", sa.Integer(), nullable=False),
        sa.Column("version_range", sa.String(length=256), nullable=True),
        sa.ForeignKeyConstraint(["deb_dependency_id"], ["deb_dependency.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["deb_package_version_id"], ["deb_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "deb_dependency_id", "deb_package_version_id"),
    )
    op.create_table(
        "deb_replaces",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("deb_dependency_id", sa.Integer(), nullable=False),
        sa.Column("deb_package_version_id", sa.Integer(), nullable=False),
        sa.Column("version_range", sa.String(length=256), nullable=False),
        sa.ForeignKeyConstraint(["deb_dependency_id"], ["deb_dependency.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["deb_package_version_id"], ["deb_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "deb_dependency_id", "deb_package_version_id"),
    )
    op.create_table(
        "dependency_monkey_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("dependency_monkey_document_id", sa.String(length=256), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("dependency_monkey_name", sa.String(length=256), nullable=False),
        sa.Column("dependency_monkey_version", sa.String(length=256), nullable=False),
        sa.Column("seed", sa.Integer(), nullable=True),
        sa.Column("decision", sa.String(length=64), nullable=False),
        sa.Column("count", sa.Integer(), nullable=True),
        sa.Column("limit_latest_versions", sa.Integer(), nullable=True),
        sa.Column("debug", sa.Boolean(), nullable=True),
        sa.Column("dependency_monkey_error", sa.Boolean(), nullable=True),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("run_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("build_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("run_hardware_information_id", sa.Integer(), nullable=True),
        sa.Column("build_hardware_information_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["build_hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["build_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["run_hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["run_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "has_symbol",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("software_environment_id", sa.Integer(), nullable=False),
        sa.Column("versioned_symbol_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["versioned_symbol_id"], ["versioned_symbol.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "software_environment_id", "versioned_symbol_id"),
    )
    op.create_table(
        "included_file",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("file", sa.String(length=256), nullable=False),
        sa.Column("python_file_digest_id", sa.Integer(), nullable=False),
        sa.Column("python_artifact_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["python_artifact_id"], ["python_artifact.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_file_digest_id"], ["python_file_digest.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_file_digest_id", "python_artifact_id"),
    )
    op.create_table(
        "package_extract_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_extract_name", sa.String(length=256), nullable=False),
        sa.Column("package_extract_version", sa.String(length=256), nullable=False),
        sa.Column("analysis_document_id", sa.String(length=256), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("environment_type", postgresql.ENUM("RUNTIME", "BUILDTIME", name="environment_type"), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=True),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("package_extract_error", sa.Boolean(), nullable=False),
        sa.Column("image_tag", sa.String(length=256), nullable=False),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("os_name", sa.String(length=256), nullable=False),
        sa.Column("os_id", sa.String(length=256), nullable=False),
        sa.Column("os_version_id", sa.String(length=256), nullable=False),
        sa.Column("software_environment_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "provenance_checker_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("provenance_checker_document_id", sa.String(length=256), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("provenance_checker_version", sa.String(length=256), nullable=False),
        sa.Column("provenance_checker_name", sa.String(length=256), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=True),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("provenance_checker_error", sa.Boolean(), nullable=False),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("user_software_stack_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["user_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "user_software_stack_id"),
    )
    op.create_table(
        "python_package_requirement",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("name", sa.String(length=256), nullable=False),
        sa.Column("version_range", sa.String(length=256), nullable=False),
        sa.Column("develop", sa.Boolean(), nullable=False),
        sa.Column("python_package_index_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["python_package_index_id"], ["python_package_index.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_package_version_entity",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_name", sa.String(length=256), nullable=False),
        sa.Column("package_version", sa.String(length=256), nullable=True),
        sa.Column("python_package_index_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["python_package_index_id"], ["python_package_index.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("id"),
        sa.UniqueConstraint("package_name", "package_version", "python_package_index_id"),
    )
    op.create_index(
        "python_package_version_entity_idx",
        "python_package_version_entity",
        ["package_name", "package_version", "python_package_index_id"],
        unique=True,
    )
    op.create_table(
        "requires_symbol",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_artifact_id", sa.Integer(), nullable=False),
        sa.Column("versioned_symbol_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["python_artifact_id"], ["python_artifact.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["versioned_symbol_id"], ["versioned_symbol.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_artifact_id", "versioned_symbol_id"),
    )
    op.create_table(
        "rpm_requires",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("rpm_package_version_id", sa.Integer(), nullable=False),
        sa.Column("rpm_requirement_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["rpm_package_version_id"], ["rpm_package_version.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["rpm_requirement_id"], ["rpm_requirement.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "rpm_package_version_id", "rpm_requirement_id"),
    )
    op.create_table(
        "advised",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("adviser_run_id", sa.Integer(), nullable=True),
        sa.Column("python_software_stack_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["adviser_run_id"], ["adviser_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "detected_symbol",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_extract_run_id", sa.Integer(), nullable=False),
        sa.Column("versioned_symbol_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["package_extract_run_id"], ["package_extract_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["versioned_symbol_id"], ["versioned_symbol.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "package_extract_run_id", "versioned_symbol_id"),
    )
    op.create_table(
        "found_deb",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("deb_package_version_id", sa.Integer(), nullable=False),
        sa.Column("package_extract_run_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["deb_package_version_id"], ["deb_package_version.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["package_extract_run_id"], ["package_extract_run.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "deb_package_version_id", "package_extract_run_id"),
    )
    op.create_table(
        "found_python_file",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("file", sa.String(length=256), nullable=False),
        sa.Column("python_file_digest_id", sa.Integer(), nullable=False),
        sa.Column("package_extract_run_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["package_extract_run_id"], ["package_extract_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_file_digest_id"], ["python_file_digest.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_file_digest_id", "package_extract_run_id"),
    )
    op.create_table(
        "found_rpm",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("rpm_package_version_id", sa.Integer(), nullable=False),
        sa.Column("package_extract_run_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["package_extract_run_id"], ["package_extract_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["rpm_package_version_id"], ["rpm_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "rpm_package_version_id", "package_extract_run_id"),
    )
    op.create_table(
        "has_vulnerability",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False),
        sa.Column("cve_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["cve_id"], ["cve.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "python_package_version_entity_id", "cve_id"),
    )
    op.create_table(
        "inspection_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_document_id", sa.String(length=256), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=True),
        sa.Column("amun_version", sa.String(length=256), nullable=True),
        sa.Column("build_requests_cpu", sa.Float(), nullable=True),
        sa.Column("build_requests_memory", sa.Float(), nullable=True),
        sa.Column("run_requests_cpu", sa.Float(), nullable=True),
        sa.Column("run_requests_memory", sa.Float(), nullable=True),
        sa.Column(
            "inspection_sync_state", postgresql.ENUM("PENDING", "SYNCED", name="inspection_sync_state"), nullable=False
        ),
        sa.Column("build_hardware_information_id", sa.Integer(), nullable=True),
        sa.Column("run_hardware_information_id", sa.Integer(), nullable=True),
        sa.Column("build_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("run_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("dependency_monkey_run_id", sa.Integer(), nullable=True),
        sa.Column("inspection_software_stack_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["build_hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["build_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["dependency_monkey_run_id"], ["dependency_monkey_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["inspection_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["run_hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["run_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("inspection_document_id"),
    )
    op.create_table(
        "package_analyzer_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_analyzer_name", sa.String(length=256), nullable=True),
        sa.Column("package_analyzer_version", sa.String(length=256), nullable=True),
        sa.Column("package_analysis_document_id", sa.String(length=256), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("package_analyzer_error", sa.Boolean(), nullable=False),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("input_python_package_version_entity_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["input_python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_dependency_monkey_requirements",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_package_requirement_id", sa.Integer(), nullable=False),
        sa.Column("dependency_monkey_run_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["dependency_monkey_run_id"], ["dependency_monkey_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["python_package_requirement_id"], ["python_package_requirement.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id", "python_package_requirement_id", "dependency_monkey_run_id"),
    )
    op.create_table(
        "python_package_version",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_name", sa.String(length=256), nullable=False),
        sa.Column("package_version", sa.String(length=256), nullable=False),
        sa.Column("os_name", sa.String(length=256), nullable=True),
        sa.Column("os_version", sa.String(length=256), nullable=True),
        sa.Column("python_version", sa.String(length=256), nullable=True),
        sa.Column("entity_id", sa.Integer(), nullable=False),
        sa.Column("python_package_index_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_package_index_id"], ["python_package_index.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint(
            "package_name", "package_version", "python_package_index_id", "os_name", "os_version", "python_version"
        ),
    )
    op.create_index(
        "python_package_version_idx",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_10",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "python_package_index_id"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_11",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "os_name"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_12",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "os_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_13",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "python_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_20",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "python_package_index_id", "os_name"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_21",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "python_package_index_id", "os_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_22",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "python_package_index_id", "python_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_23",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "os_name", "os_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_24",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "os_name", "python_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_25",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "os_version", "python_version"],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_30",
        "python_package_version",
        [
            "package_name",
            "package_version",
            "python_package_index_id",
            "python_package_index_id",
            "os_name",
            "os_version",
        ],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_31",
        "python_package_version",
        [
            "package_name",
            "package_version",
            "python_package_index_id",
            "python_package_index_id",
            "os_name",
            "python_version",
        ],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_32",
        "python_package_version",
        [
            "package_name",
            "package_version",
            "python_package_index_id",
            "python_package_index_id",
            "os_version",
            "python_version",
        ],
        unique=False,
    )
    op.create_index(
        "python_package_version_index_idx_33",
        "python_package_version",
        ["package_name", "package_version", "python_package_index_id", "os_name", "os_version", "python_version"],
        unique=False,
    )
    op.create_table(
        "python_requirements",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_package_requirement_id", sa.Integer(), nullable=False),
        sa.Column("python_software_stack_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["python_package_requirement_id"], ["python_package_requirement.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["python_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_package_requirement_id", "python_software_stack_id"),
    )
    op.create_table(
        "depends_on",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("entity_id", sa.Integer(), nullable=False),
        sa.Column("version_id", sa.Integer(), nullable=False),
        sa.Column("version_range", sa.String(length=128), nullable=True),
        sa.ForeignKeyConstraint(["entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["version_id"], ["python_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "entity_id", "version_id"),
        sa.UniqueConstraint("entity_id", "version_id", "version_range"),
    )
    op.create_index("depends_on_idx", "depends_on", ["entity_id", "version_id", "version_range"], unique=False)
    op.create_index(
        "depends_on_version_range_idx", "depends_on", ["entity_id", "version_id", "version_range"], unique=False
    )
    op.create_table(
        "has_artifact",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_package_version_id", sa.Integer(), nullable=False),
        sa.Column("python_artifact_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["python_artifact_id"], ["python_artifact.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_package_version_id"], ["python_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_package_version_id", "python_artifact_id"),
    )
    op.create_table(
        "identified",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_extract_run_id", sa.Integer(), nullable=False),
        sa.Column("python_package_version_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["package_extract_run_id"], ["package_extract_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_package_version_id"], ["python_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "package_extract_run_id", "python_package_version_id"),
    )
    op.create_table(
        "investigated",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_analyzer_run_id", sa.Integer(), nullable=False),
        sa.Column("python_artifact_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["package_analyzer_run_id"], ["package_analyzer_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_artifact_id"], ["python_artifact.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "package_analyzer_run_id", "python_artifact_id"),
    )
    op.create_table(
        "investigated_file",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("package_analyzer_run_id", sa.Integer(), nullable=False),
        sa.Column("python_file_digest_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["package_analyzer_run_id"], ["package_analyzer_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_file_digest_id"], ["python_file_digest.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "package_analyzer_run_id", "python_file_digest_id"),
    )
    op.create_table(
        "pi_conv1d",
        sa.Column("framework", sa.String(length=256), nullable=True),
        sa.Column("origin", sa.String(length=256), nullable=True),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("device", sa.String(length=256), nullable=False),
        sa.Column("dtype", sa.String(length=256), nullable=False),
        sa.Column("reps", sa.Integer(), nullable=False),
        sa.Column("data_format", sa.String(length=256), nullable=False),
        sa.Column("batch", sa.Integer(), nullable=False),
        sa.Column("input_width", sa.Integer(), nullable=False),
        sa.Column("input_channels", sa.Integer(), nullable=False),
        sa.Column("filter_width", sa.Integer(), nullable=False),
        sa.Column("output_channels", sa.Integer(), nullable=False),
        sa.Column("strides", sa.Integer(), nullable=False),
        sa.Column("padding", sa.String(length=256), nullable=False),
        sa.Column("elapsed", sa.Float(), nullable=False),
        sa.Column("rate", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "pi_conv2d",
        sa.Column("framework", sa.String(length=256), nullable=True),
        sa.Column("origin", sa.String(length=256), nullable=True),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("device", sa.String(length=256), nullable=False),
        sa.Column("dtype", sa.String(length=256), nullable=False),
        sa.Column("reps", sa.Integer(), nullable=False),
        sa.Column("data_format", sa.String(length=256), nullable=False),
        sa.Column("batch", sa.Integer(), nullable=False),
        sa.Column("input_height", sa.Integer(), nullable=False),
        sa.Column("input_width", sa.Integer(), nullable=False),
        sa.Column("input_channels", sa.Integer(), nullable=False),
        sa.Column("filter_height", sa.Integer(), nullable=False),
        sa.Column("filter_width", sa.Integer(), nullable=False),
        sa.Column("output_channels", sa.Integer(), nullable=False),
        sa.Column("strides", sa.Integer(), nullable=False),
        sa.Column("padding", sa.String(length=256), nullable=False),
        sa.Column("elapsed", sa.Float(), nullable=False),
        sa.Column("rate", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "pi_matmul",
        sa.Column("framework", sa.String(length=256), nullable=True),
        sa.Column("origin", sa.String(length=256), nullable=True),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("device", sa.String(length=256), nullable=False),
        sa.Column("matrix_size", sa.Integer(), nullable=False),
        sa.Column("dtype", sa.String(length=256), nullable=False),
        sa.Column("reps", sa.Integer(), nullable=False),
        sa.Column("elapsed", sa.Float(), nullable=False),
        sa.Column("rate", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_requirements_lock",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("python_package_version_id", sa.Integer(), nullable=False),
        sa.Column("python_software_stack_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["python_package_version_id"], ["python_package_version.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "python_package_version_id", "python_software_stack_id"),
    )
    op.create_table(
        "solved",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("document_id", sa.String(length=128), nullable=False),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("error", sa.Boolean(), nullable=False),
        sa.Column("error_unparseable", sa.Boolean(), nullable=False),
        sa.Column("error_unsolvable", sa.Boolean(), nullable=False),
        sa.Column("is_provided", sa.Boolean(), nullable=True),
        sa.Column("ecosystem_solver_id", sa.Integer(), nullable=False),
        sa.Column("version_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["ecosystem_solver_id"], ["ecosystem_solver.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["version_id"], ["python_package_version.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "ecosystem_solver_id", "version_id"),
    )
    op.create_index("solver_document_id_idx", "solved", ["document_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("solver_document_id_idx", table_name="solved")
    op.drop_table("solved")
    op.drop_table("python_requirements_lock")
    op.drop_table("pi_matmul")
    op.drop_table("pi_conv2d")
    op.drop_table("pi_conv1d")
    op.drop_table("investigated_file")
    op.drop_table("investigated")
    op.drop_table("identified")
    op.drop_table("has_artifact")
    op.drop_index("depends_on_version_range_idx", table_name="depends_on")
    op.drop_index("depends_on_idx", table_name="depends_on")
    op.drop_table("depends_on")
    op.drop_table("python_requirements")
    op.drop_index("python_package_version_index_idx_33", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_32", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_31", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_30", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_25", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_24", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_23", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_22", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_21", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_20", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_13", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_12", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_11", table_name="python_package_version")
    op.drop_index("python_package_version_index_idx_10", table_name="python_package_version")
    op.drop_index("python_package_version_idx", table_name="python_package_version")
    op.drop_table("python_package_version")
    op.drop_table("python_dependency_monkey_requirements")
    op.drop_table("package_analyzer_run")
    op.drop_table("inspection_run")
    op.drop_table("has_vulnerability")
    op.drop_table("found_rpm")
    op.drop_table("found_python_file")
    op.drop_table("found_deb")
    op.drop_table("detected_symbol")
    op.drop_table("advised")
    op.drop_table("rpm_requires")
    op.drop_table("requires_symbol")
    op.drop_index("python_package_version_entity_idx", table_name="python_package_version_entity")
    op.drop_table("python_package_version_entity")
    op.drop_table("python_package_requirement")
    op.drop_table("provenance_checker_run")
    op.drop_table("package_extract_run")
    op.drop_table("included_file")
    op.drop_table("has_symbol")
    op.drop_table("dependency_monkey_run")
    op.drop_table("deb_replaces")
    op.drop_table("deb_pre_depends")
    op.drop_table("deb_depends")
    op.drop_table("adviser_run")
    op.drop_table("versioned_symbol")
    op.drop_table("software_environment")
    op.drop_table("rpm_requirement")
    op.drop_table("rpm_package_version")
    op.drop_table("python_software_stack")
    op.drop_index("url_idx", table_name="python_package_index")
    op.drop_table("python_package_index")
    op.drop_index("sha256_idx", table_name="python_file_digest")
    op.drop_table("python_file_digest")
    op.drop_table("python_artifact")
    op.drop_table("hardware_information")
    op.drop_index("ecosystem_solver_idx", table_name="ecosystem_solver")
    op.drop_table("ecosystem_solver")
    op.drop_table("deb_package_version")
    op.drop_table("deb_dependency")
    op.drop_table("cve")
    # ### end Alembic commands ###



"""Added image_size attribute to PackageExtractRun

Revision ID: dd19dd789f6d
Revises: a05ad6e1d0ca
Create Date: 2019-10-29 09:07:00.504320+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "dd19dd789f6d"
down_revision = "a05ad6e1d0ca"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("package_extract_run", sa.Column("image_size", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("package_extract_run", "image_size")
    # ### end Alembic commands ###



"""Remove string size limitation from marker and extras

Revision ID: 9fea7ca59f56
Revises: fd7771734460
Create Date: 2020-01-03 11:41:38.419935+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9fea7ca59f56"
down_revision = "fd7771734460"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("depends_on", "marker", type_=sa.Text, existing_type=sa.VARCHAR)
    op.alter_column("depends_on", "extra", type_=sa.Text, existing_type=sa.VARCHAR)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("depends_on", "marker", type_=sa.String, existing_type=sa.VARCHAR)
    op.alter_column("depends_on", "extra", type_=sa.String, existing_type=sa.VARCHAR)
    # ### end Alembic commands ###



"""Added new table to link adviser run and python package version entity

Revision ID: 74254b596f00
Revises: ed56455efd82
Create Date: 2020-03-20 15:31:37.608247+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "74254b596f00"
down_revision = "ed56455efd82"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "has_unresolved",
        sa.Column("adviser_run_id", sa.Integer(), nullable=False),
        sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["adviser_run_id"], ["adviser_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("adviser_run_id", "python_package_version_entity_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("has_unresolved")
    # ### end Alembic commands ###



"""Refactor to have correct Schema for Python Software Stack

Revision ID: 2d7ef94ff4dd
Revises: 1a8150ac45e0
Create Date: 2020-10-09 16:04:29.846495+00:00

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "2d7ef94ff4dd"
down_revision = "1a8150ac45e0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "external_python_requirements",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("requirements_hash", sa.Text(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "external_python_requirements_lock",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("requirements_lock_hash", sa.Text(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_requirements",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("requirements_hash", sa.Text(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_requirements_lock",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("requirements_lock_hash", sa.Text(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "external_python_software_stack",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("external_python_requirements_id", sa.Integer(), nullable=True),
        sa.Column("external_python_requirements_lock_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["external_python_requirements_id"], ["external_python_requirements.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["external_python_requirements_lock_id"], ["external_python_requirements_lock.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "python_software_stack",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column(
            "software_stack_type",
            postgresql.ENUM("INSPECTION", "ADVISED", name="software_stack_type", create_type=False),
            nullable=True,
        ),
        sa.Column("performance_score", sa.Float(), nullable=True),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("python_requirements_id", sa.Integer(), nullable=True),
        sa.Column("python_requirements_lock_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["python_requirements_id"], ["python_requirements.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_requirements_lock_id"], ["python_requirements_lock.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "adviser_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("adviser_document_id", sa.Text(), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("adviser_version", sa.Text(), nullable=False),
        sa.Column("adviser_name", sa.Text(), nullable=False),
        sa.Column("count", sa.Integer(), nullable=True),
        sa.Column("limit", sa.Integer(), nullable=True),
        sa.Column("origin", sa.Text(), nullable=True),
        sa.Column(
            "source_type",
            postgresql.ENUM(
                "CLI", "KEBECHET", "S2I", "GITHUB_APP", "JUPYTER_NOTEBOOK", name="source_type", create_type=False
            ),
            nullable=True,
        ),
        sa.Column("is_s2i", sa.Boolean(), nullable=True),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("need_re_run", sa.Boolean(), nullable=True),
        sa.Column("re_run_adviser_id", sa.Text(), nullable=True),
        sa.Column("limit_latest_versions", sa.Integer(), nullable=True),
        sa.Column("adviser_error", sa.Boolean(), nullable=False),
        sa.Column(
            "recommendation_type",
            postgresql.ENUM("STABLE", "TESTING", "LATEST", name="recommendation_type", create_type=False),
            nullable=False,
        ),
        sa.Column(
            "requirements_format",
            postgresql.ENUM("PIPENV", name="requirements_format", create_type=False),
            nullable=False,
        ),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("advised_configuration_changes", sa.Boolean(), nullable=False),
        sa.Column("additional_stack_info", sa.Boolean(), nullable=False),
        sa.Column("user_software_stack_id", sa.Integer(), nullable=True),
        sa.Column("external_run_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("external_build_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("external_hardware_information_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["external_build_software_environment_id"], ["external_software_environment.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["external_hardware_information_id"], ["external_hardware_information.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["external_run_software_environment_id"], ["external_software_environment.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["user_software_stack_id"], ["external_python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "has_external_python_requirement",
        sa.Column("external_python_requirements_id", sa.Integer(), nullable=False),
        sa.Column("python_package_requirement_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["external_python_requirements_id"], ["external_python_requirements.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["python_package_requirement_id"], ["python_package_requirement.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("external_python_requirements_id", "python_package_requirement_id"),
    )
    op.create_table(
        "has_external_python_requirement_lock",
        sa.Column("external_python_requirements_locked_id", sa.Integer(), nullable=False),
        sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["external_python_requirements_locked_id"], ["external_python_requirements_lock.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("external_python_requirements_locked_id", "python_package_version_entity_id"),
    )
    op.create_table(
        "has_python_requirements",
        sa.Column("python_requirements_id", sa.Integer(), nullable=False),
        sa.Column("python_package_requirement_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["python_package_requirement_id"], ["python_package_requirement.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["python_requirements_id"], ["python_requirements.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("python_requirements_id", "python_package_requirement_id"),
    )
    op.create_table(
        "inspection_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_document_id", sa.Text(), nullable=False),
        sa.Column("inspection_result_number", sa.Integer(), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=True),
        sa.Column("amun_version", sa.Text(), nullable=True),
        sa.Column("build_requests_cpu", sa.Float(), nullable=True),
        sa.Column("build_requests_memory", sa.Float(), nullable=True),
        sa.Column("run_requests_cpu", sa.Float(), nullable=True),
        sa.Column("run_requests_memory", sa.Float(), nullable=True),
        sa.Column(
            "inspection_sync_state",
            postgresql.ENUM("PENDING", "SYNCED", name="inspection_sync_state", create_type=False),
            nullable=False,
        ),
        sa.Column("build_hardware_information_id", sa.Integer(), nullable=True),
        sa.Column("run_hardware_information_id", sa.Integer(), nullable=True),
        sa.Column("build_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("run_software_environment_id", sa.Integer(), nullable=True),
        sa.Column("dependency_monkey_run_id", sa.Integer(), nullable=True),
        sa.Column("inspection_software_stack_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(["build_hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["build_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["dependency_monkey_run_id"], ["dependency_monkey_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["inspection_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["run_hardware_information_id"], ["hardware_information.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["run_software_environment_id"], ["software_environment.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "provenance_checker_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("provenance_checker_document_id", sa.Text(), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("provenance_checker_version", sa.Text(), nullable=False),
        sa.Column("provenance_checker_name", sa.Text(), nullable=False),
        sa.Column("origin", sa.Text(), nullable=True),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("provenance_checker_error", sa.Boolean(), nullable=False),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("user_software_stack_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["user_software_stack_id"], ["external_python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id", "user_software_stack_id"),
    )
    op.create_table(
        "advised",
        sa.Column("adviser_run_id", sa.Integer(), nullable=False),
        sa.Column("python_software_stack_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["adviser_run_id"], ["adviser_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_software_stack_id"], ["python_software_stack.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("adviser_run_id", "python_software_stack_id"),
    )
    op.create_table(
        "has_python_requirement_lock",
        sa.Column("python_requirements_lock_id", sa.Integer(), nullable=False),
        sa.Column("python_package_version_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["python_package_version_id"], ["python_package_version.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["python_requirements_lock_id"], ["python_requirements_lock.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("python_requirements_lock_id", "python_package_version_id"),
    )
    op.create_table(
        "has_unresolved",
        sa.Column("adviser_run_id", sa.Integer(), nullable=False),
        sa.Column("python_package_version_entity_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["adviser_run_id"], ["adviser_run.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(
            ["python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("adviser_run_id", "python_package_version_entity_id"),
    )
    op.create_table(
        "pi_conv1d",
        sa.Column("component", sa.String(length=256), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=False),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("device", sa.String(length=256), nullable=False),
        sa.Column("dtype", sa.String(length=256), nullable=False),
        sa.Column("reps", sa.Integer(), nullable=False),
        sa.Column("data_format", sa.String(length=256), nullable=False),
        sa.Column("batch", sa.Integer(), nullable=False),
        sa.Column("input_width", sa.Integer(), nullable=False),
        sa.Column("input_channels", sa.Integer(), nullable=False),
        sa.Column("filter_width", sa.Integer(), nullable=False),
        sa.Column("output_channels", sa.Integer(), nullable=False),
        sa.Column("strides", sa.Integer(), nullable=False),
        sa.Column("padding", sa.Integer(), nullable=False),
        sa.Column("elapsed", sa.Float(), nullable=False),
        sa.Column("rate", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "pi_conv2d",
        sa.Column("component", sa.String(length=256), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=False),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("device", sa.String(length=256), nullable=False),
        sa.Column("dtype", sa.String(length=256), nullable=False),
        sa.Column("reps", sa.Integer(), nullable=False),
        sa.Column("data_format", sa.String(length=256), nullable=False),
        sa.Column("batch", sa.Integer(), nullable=False),
        sa.Column("input_height", sa.Integer(), nullable=False),
        sa.Column("input_width", sa.Integer(), nullable=False),
        sa.Column("input_channels", sa.Integer(), nullable=False),
        sa.Column("filter_height", sa.Integer(), nullable=False),
        sa.Column("filter_width", sa.Integer(), nullable=False),
        sa.Column("output_channels", sa.Integer(), nullable=False),
        sa.Column("strides", sa.Integer(), nullable=False),
        sa.Column("padding", sa.Integer(), nullable=False),
        sa.Column("elapsed", sa.Float(), nullable=False),
        sa.Column("rate", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "pi_matmul",
        sa.Column("component", sa.String(length=256), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=False),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("device", sa.String(length=256), nullable=False),
        sa.Column("matrix_size", sa.Integer(), nullable=False),
        sa.Column("dtype", sa.String(length=256), nullable=False),
        sa.Column("reps", sa.Integer(), nullable=False),
        sa.Column("elapsed", sa.Float(), nullable=False),
        sa.Column("rate", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "pi_pybench",
        sa.Column("component", sa.String(length=256), nullable=False),
        sa.Column("origin", sa.String(length=256), nullable=False),
        sa.Column("version", sa.String(length=256), nullable=False),
        sa.Column("overall_score", sa.Float(), nullable=True),
        sa.Column("exit_code", sa.Integer(), nullable=False),
        sa.Column("ru_utime", sa.Float(), nullable=False),
        sa.Column("ru_stime", sa.Float(), nullable=False),
        sa.Column("ru_maxrss", sa.Integer(), nullable=False),
        sa.Column("ru_ixrss", sa.Integer(), nullable=False),
        sa.Column("ru_idrss", sa.Integer(), nullable=False),
        sa.Column("ru_isrss", sa.Integer(), nullable=False),
        sa.Column("ru_minflt", sa.Integer(), nullable=False),
        sa.Column("ru_majflt", sa.Integer(), nullable=False),
        sa.Column("ru_nswap", sa.Integer(), nullable=False),
        sa.Column("ru_inblock", sa.Integer(), nullable=False),
        sa.Column("ru_oublock", sa.Integer(), nullable=False),
        sa.Column("ru_msgsnd", sa.Integer(), nullable=False),
        sa.Column("ru_msgrcv", sa.Integer(), nullable=False),
        sa.Column("ru_nsignals", sa.Integer(), nullable=False),
        sa.Column("ru_nvcsw", sa.Integer(), nullable=False),
        sa.Column("ru_nivcsw", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("inspection_run_id", sa.Integer(), nullable=False),
        sa.Column("rounds", sa.Integer(), nullable=False),
        sa.Column("built_in_function_calls_average", sa.Float(), nullable=False),
        sa.Column("built_in_method_lookup_average", sa.Float(), nullable=False),
        sa.Column("compare_floats_average", sa.Float(), nullable=False),
        sa.Column("compare_floats_integers_average", sa.Float(), nullable=False),
        sa.Column("compare_integers_average", sa.Float(), nullable=False),
        sa.Column("compare_interned_strings_average", sa.Float(), nullable=False),
        sa.Column("compare_longs_average", sa.Float(), nullable=False),
        sa.Column("compare_strings_average", sa.Float(), nullable=False),
        sa.Column("compare_unicode_average", sa.Float(), nullable=False),
        sa.Column("concat_strings_average", sa.Float(), nullable=False),
        sa.Column("concat_unicode_average", sa.Float(), nullable=False),
        sa.Column("create_instances_average", sa.Float(), nullable=False),
        sa.Column("create_new_instances_average", sa.Float(), nullable=False),
        sa.Column("create_strings_with_concat_average", sa.Float(), nullable=False),
        sa.Column("create_unicode_with_concat_average", sa.Float(), nullable=False),
        sa.Column("dict_creation_average", sa.Float(), nullable=False),
        sa.Column("dict_with_float_keys_average", sa.Float(), nullable=False),
        sa.Column("dict_with_integer_keys_average", sa.Float(), nullable=False),
        sa.Column("dict_with_string_keys_average", sa.Float(), nullable=False),
        sa.Column("for_loops_average", sa.Float(), nullable=False),
        sa.Column("if_then_else_average", sa.Float(), nullable=False),
        sa.Column("list_slicing_average", sa.Float(), nullable=False),
        sa.Column("nested_for_loops_average", sa.Float(), nullable=False),
        sa.Column("normal_class_attribute_average", sa.Float(), nullable=False),
        sa.Column("normal_instance_attribute_average", sa.Float(), nullable=False),
        sa.Column("python_function_calls_average", sa.Float(), nullable=False),
        sa.Column("python_method_calls_average", sa.Float(), nullable=False),
        sa.Column("recursion_average", sa.Float(), nullable=False),
        sa.Column("second_import_average", sa.Float(), nullable=False),
        sa.Column("second_package_import_average", sa.Float(), nullable=False),
        sa.Column("second_submodule_import_average", sa.Float(), nullable=False),
        sa.Column("simple_complex_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_dict_manipulation_average", sa.Float(), nullable=False),
        sa.Column("simple_float_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_int_float_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_integer_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("simple_list_manipulation_average", sa.Float(), nullable=False),
        sa.Column("simple_long_arithmetic_average", sa.Float(), nullable=False),
        sa.Column("small_lists_average", sa.Float(), nullable=False),
        sa.Column("small_tuples_average", sa.Float(), nullable=False),
        sa.Column("special_class_attribute_average", sa.Float(), nullable=False),
        sa.Column("special_instance_attribute_average", sa.Float(), nullable=False),
        sa.Column("string_mappings_average", sa.Float(), nullable=False),
        sa.Column("string_predicates_average", sa.Float(), nullable=False),
        sa.Column("string_slicing_average", sa.Float(), nullable=False),
        sa.Column("try_except_average", sa.Float(), nullable=False),
        sa.Column("try_raise_except_average", sa.Float(), nullable=False),
        sa.Column("tuple_slicing_average", sa.Float(), nullable=False),
        sa.Column("unicode_mappings_average", sa.Float(), nullable=False),
        sa.Column("unicode_predicates_average", sa.Float(), nullable=False),
        sa.Column("unicode_properties_average", sa.Float(), nullable=False),
        sa.Column("unicode_slicing_average", sa.Float(), nullable=False),
        sa.Column("totals_average", sa.Float(), nullable=False),
        sa.ForeignKeyConstraint(
            ["inspection_run_id"],
            ["inspection_run.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("pi_pybench")
    op.drop_table("pi_matmul")
    op.drop_table("pi_conv2d")
    op.drop_table("pi_conv1d")
    op.drop_table("has_unresolved")
    op.drop_table("has_python_requirement_lock")
    op.drop_table("advised")
    op.drop_table("provenance_checker_run")
    op.drop_table("inspection_run")
    op.drop_table("has_python_requirements")
    op.drop_table("has_external_python_requirement_lock")
    op.drop_table("has_external_python_requirement")
    op.drop_table("adviser_run")
    op.drop_table("python_software_stack")
    op.drop_table("external_python_software_stack")
    op.drop_table("python_requirements_lock")
    op.drop_table("python_requirements")
    op.drop_table("external_python_requirements_lock")
    op.drop_table("external_python_requirements")
    # ### end Alembic commands ###



"""Add indexes for abi_compatability queries

Revision ID: 0c8eab5d4197
Revises: bb26306538f0
Create Date: 2020-02-05 10:54:52.119190+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "0c8eab5d4197"
down_revision = "bb26306538f0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "build_log_analyzer_run",
        sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
        sa.Column("build_log_analyzer_name", sa.Text(), nullable=True),
        sa.Column("build_log_analyzer_version", sa.Text(), nullable=True),
        sa.Column("build_log_analysis_document_id", sa.Text(), nullable=False),
        sa.Column("datetime", sa.DateTime(), nullable=False),
        sa.Column("debug", sa.Boolean(), nullable=False),
        sa.Column("build_log_analyzer_error_reason", sa.Text(), nullable=True),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("input_python_package_version_entity_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["input_python_package_version_entity_id"], ["python_package_version_entity.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index("python_package_version_entity_id_idx", "python_package_version_entity", ["id"], unique=True)
    op.create_index("requires_symbol_python_artifact_id_idx", "requires_symbol", ["python_artifact_id"], unique=False)
    op.create_index("versioned_symbol_id_idx", "versioned_symbol", ["id"], unique=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("versioned_symbol_id_idx", table_name="versioned_symbol")
    op.drop_index("requires_symbol_python_artifact_id_idx", table_name="requires_symbol")
    op.drop_index("python_package_version_entity_id_idx", table_name="python_package_version_entity")
    op.drop_table("build_log_analyzer_run")
    # ### end Alembic commands ###



"""Drop id on relation tables

Revision ID: d0ebc816243c
Revises: af2b9e09e53e
Create Date: 2020-01-08 13:36:04.822647+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "d0ebc816243c"
down_revision = "af2b9e09e53e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("advised", "id")
    op.drop_column("deb_pre_depends", "id")
    op.drop_column("deb_replaces", "id")
    op.drop_column("depends_on", "id")
    op.drop_column("detected_symbol", "id")
    op.drop_column("found_deb", "id")
    op.drop_column("found_python_file", "id")
    op.drop_column("found_python_interpreter", "id")
    op.drop_column("found_rpm", "id")
    op.drop_column("has_artifact", "id")
    op.drop_column("has_metadata_classifier", "id")
    op.drop_column("has_metadata_distutils", "id")
    op.drop_column("has_metadata_platform", "id")
    op.drop_column("has_metadata_project_url", "id")
    op.drop_column("has_metadata_provides_extra", "id")
    op.drop_column("has_metadata_requires_external", "id")
    op.drop_column("has_metadata_supported_platform", "id")
    op.drop_column("has_symbol", "id")
    op.drop_column("has_vulnerability", "id")
    op.drop_column("identified", "id")
    op.drop_column("included_file", "id")
    op.drop_column("investigated", "id")
    op.drop_column("investigated_file", "id")
    op.drop_column("requires_symbol", "id")
    op.drop_column("rpm_requires", "id")
    op.drop_column("solved", "id")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("solved", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("rpm_requires", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("requires_symbol", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("investigated_file", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("investigated", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("included_file", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("identified", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_vulnerability", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_symbol", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_supported_platform", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_requires_external", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_provides_extra", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_project_url", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_platform", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_distutils", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_metadata_classifier", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("has_artifact", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("found_rpm", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("found_python_interpreter", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("found_python_file", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("found_deb", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("detected_symbol", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("depends_on", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("deb_replaces", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("deb_pre_depends", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    op.add_column("advised", sa.Column("id", sa.INTEGER(), autoincrement=True, nullable=False))
    # ### end Alembic commands ###



import os
import sys
from setuptools import setup
from setuptools.command.test import test as TestCommand
from pathlib import Path


def get_install_requires():
    with open("requirements.txt", "r") as requirements_file:
        # TODO: respect hashes in requirements.txt file
        res = requirements_file.readlines()
        return [req.split(" ", maxsplit=1)[0] for req in res if req]


def get_test_requires():
    with open("requirements-test.txt", "r") as requirements_file:
        res = requirements_file.readlines()
        return [req.split(" ", maxsplit=1)[0] for req in res if req]


def get_version():
    with open(os.path.join("thoth", "storages", "__init__.py")) as f:
        content = f.readlines()

    for line in content:
        if line.startswith("__version__ ="):
            # dirty, remove trailing and leading chars
            return line.split(" = ")[1][1:-2]
    raise ValueError("No version identifier found")


class Test(TestCommand):
    """Introduce test command to run testsuite using pytest."""

    _IMPLICIT_PYTEST_ARGS = [
        "tests/",
        "--timeout=2",
        "--cov=./thoth",
        # '--mypy',
        "thoth/",
        "--capture=no",
        "--verbose",
        "-l",
        "-s",
        "-vv",
    ]

    user_options = [("pytest-args=", "a", "Arguments to pass into py.test")]

    def initialize_options(self):
        super().initialize_options()
        self.pytest_args = None

    def finalize_options(self):
        super().finalize_options()
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        passed_args = list(self._IMPLICIT_PYTEST_ARGS)

        if self.pytest_args:
            self.pytest_args = [arg for arg in self.pytest_args.split() if arg]
            passed_args.extend(self.pytest_args)

        sys.exit(pytest.main(passed_args))


VERSION = get_version()
setup(
    name="thoth-storages",
    version=VERSION,
    description="Storage and database adapters available in project Thoth",
    long_description=Path("README.rst").read_text(),
    long_description_content_type="text/x-rst",
    author="Fridolin Pokorny",
    author_email="fridolin@redhat.com",
    license="GPLv3+",
    packages=["thoth.storages", "thoth.storages.graph"],
    package_data={
        "thoth.storages": [
            "py.typed",
            os.path.join("data", "alembic.ini"),
            os.path.join("data", "alembic", "script.py.mako"),
            os.path.join("data", "alembic", "env.py"),
            os.path.join("data", "alembic", "versions", "*.py"),
        ]
    },
    zip_safe=False,
    install_requires=get_install_requires(),
    tests_require=get_test_requires(),
    cmdclass={"test": Test},
    entry_points={"console_scripts": ["thoth-storages=thoth.storages.cli:cli"]},
    url="https://github.com/thoth-station/storages",
    maintainer="Francesco Murdaca",
    maintainer_email="fmurdaca@redhat.com",
    command_options={
        "build_sphinx": {
            "version": ("setup.py", VERSION),
            "release": ("setup.py", VERSION),
        }
    },
)



"""Add datetime for filtering to SecurityIndicatorAggregatedRun

Revision ID: 10925df620b1
Revises: 9aa4e29fa260
Create Date: 2020-07-07 10:46:35.292208+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "10925df620b1"
down_revision = "9aa4e29fa260"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("si_aggregated_run", sa.Column("datetime", sa.DateTime(), nullable=False))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("si_aggregated_run", "datetime")
    # ### end Alembic commands ###



"""Added a relation between PythonPackageVersion and SIAggregated

Revision ID: 84588707e6ba
Revises: 83e2900c3721
Create Date: 2020-08-26 18:55:20.736024+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "84588707e6ba"
down_revision = "83e2900c3721"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("si_aggregated", sa.Column("python_package_version_id", sa.Integer(), nullable=False))
    op.create_foreign_key(
        None, "si_aggregated", "python_package_version", ["python_package_version_id"], ["id"], ondelete="CASCADE"
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "si_aggregated", type_="foreignkey")
    op.drop_column("si_aggregated", "python_package_version_id")
    # ### end Alembic commands ###



"""Add result number to inspectionRun

Revision ID: d6d0b20ec650
Revises: 10925df620b1
Create Date: 2020-07-08 16:07:37.994149+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "d6d0b20ec650"
down_revision = "10925df620b1"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "inspection_run", sa.Column("inspection_result_number", sa.Integer(), nullable=False, server_default="0")
    )
    op.drop_constraint("inspection_run_inspection_document_id_key", "inspection_run", type_="unique")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "inspection_run_inspection_document_id_key", "inspection_run", ["inspection_document_id"]
    )
    op.drop_column("inspection_run", "inspection_result_number")
    # ### end Alembic commands ###



"""Provide is_s2i flag for adviser runs

Revision ID: fcac91b2a3f1
Revises: 9b210f8d5c24
Create Date: 2019-12-07 13:39:32.620452+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "fcac91b2a3f1"
down_revision = "9b210f8d5c24"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("adviser_run", sa.Column("is_s2i", sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("adviser_run", "is_s2i")
    # ### end Alembic commands ###
